from PyQt5.QtWidgets import QPushButton, QGridLayout, QWidget, QVBoxLayout
from PyQt5.QtCore import QTimer
from PlottingWindow import PlotCanvas
from LoggingSelectionMenu import LoggingSelectionMenu
from queue import Queue
import queue
from CrazyflieProtoConnection import CrazyflieProtoConnection
from SetpointMenu import SetpointMenu
from SetpointHandler import SetpointHandler
from cfclient.utils.input import JoystickReader


class ControlWindow(QWidget):
    def __init__(self, logging_queue: Queue,
                 setpoint_handler: SetpointHandler, joystick: JoystickReader,
                 cf: CrazyflieProtoConnection):
        super().__init__()

        self.logging_queue = logging_queue
        self.graph_data = False

        # Create the plot widget
        self.cf = cf

        layout = QGridLayout()
        left_side_vertical_layout = QVBoxLayout()

        self.canvas = PlotCanvas()
        self.setpoint_menu = SetpointMenu(setpoint_handler, joystick)
        self.logging_menu = LoggingSelectionMenu()

        win2 = QWidget()
        win2.setLayout(left_side_vertical_layout)
        win2.setMaximumWidth(300)

        layout.addWidget(win2, 1, 1)
        layout.addWidget(self.canvas, 1, 2)

        self.play_or_pause_logging_button = \
            QPushButton("Begin Graphing Logging")
        self.play_or_pause_logging_button.clicked.connect(
            self.play_pause_button_callback)

        left_side_vertical_layout.addWidget(self.setpoint_menu, 1)
        left_side_vertical_layout.addWidget(
            self.play_or_pause_logging_button, 2)
        left_side_vertical_layout.addWidget(self.logging_menu, 3)

        # Set up the timer to update the plot
        self.timer = QTimer()
        self.timer.timeout.connect(self.update_plot_outer)
        self.timer.start(50)

        # Set the window properties
        self.setLayout(layout)
        self.setWindowTitle("Sine Wave Plot")
        self.setGeometry(100, 100, 800, 600)
        self.show()

    def update_plot_outer(self):
        not_empty = True
        while not_empty:
            try:
                value = self.logging_queue.get_nowait()
                data = value['data']
                timestamp = value['timestamp']
                axis = self.logging_menu.get_axis_of_signal(value['signal'])

                if axis is not None and self.graph_data:
                    self.canvas.update_plot(data, timestamp, axis)

            except queue.Empty:  # done emptying logging queue

                not_empty = False

    def play_pause_button_callback(self):
        print("Hello world!")

        if self.graph_data:
            self.play_or_pause_logging_button.setText(
                "Continue Graphing Logging")
            self.graph_data = False
            self.cf.stop_logging()
        else:
            self.play_or_pause_logging_button.setText("Pause Logging")
            self.graph_data = True
            self.cf.start_logging()
