from time import time
from typing import List
import time

import cflib.crtp
from cflib.crazyflie import Crazyflie
from cflib.crazyflie.syncCrazyflie import SyncCrazyflie
from queue import Queue

from cflib.crazyflie.log import LogConfig


class CrazyflieProtoConnection:
    def __init__(self):
        # Set up the timer to update the plot
        self.start_time = time.time()

        self.logging_queue = Queue()

        self.scf = None
        self.is_connected = None
        self.param_callback_count = 0
        self.logging_configs = []

    def logging_callback(self, _timestamp, data, _logconf):

        timestamp1 = time.time() - self.start_time

        for key in data.keys():
            value_pair = {'timestamp': timestamp1, 'data': data[key],
                          'signal': key}
            self.logging_queue.put(value_pair)

    def param_set_value(self, group: str, name: str, value: float):
        try:
            if self.scf.is_link_open():
                full_name = group + "." + name
                cf = self.scf.cf
                cf.param.add_update_callback(group=group, name=name,
                                             cb=self.done_setting_param_value)

                cf.param.set_value(full_name, value)

                # Don't return until the parameter is done getting set
                while self.param_callback_count < 1:
                    time.sleep(0.01)

                self.param_callback_count = 0
        except AttributeError:
            print("Nothing connected")

    def done_setting_param_value(self, *_args):
        print("Done setting param")
        self.param_callback_count += 1

    def param_get_value(self, group: str, name: str):

        try:
            if self.scf.is_link_open():
                return self.scf.cf.param.values[group][name]
        except AttributeError:
            pass
        return -1.234567890

    def get_logging_toc(self):
        try:
            if self.scf.is_link_open():
                tocFull = self.scf.cf.log.toc.toc
                toc = []
                for key in tocFull.keys():
                    for inner_key in tocFull[key].keys():
                        full_name = key + "." + inner_key
                        toc.append(full_name)

                return toc
            else:
                return {}
        except AttributeError:
            pass
        return {}

    def get_param_toc(self):
        try:
            if self.scf.is_link_open():
                toc = self.scf.cf.param.values

                return toc
        except AttributeError:
            pass
        return {}

    def add_log_config(self, name: str, period: int, variables: List[str]):
        print("Name: " + name + ", period: " + str(period) + ", variables: "
              + str(variables))
        logging_group = LogConfig(name=name, period_in_ms=period)

        for variable in variables:
            logging_group.add_variable(variable, 'float')

        self.logging_configs.append(logging_group)
        self.logging_configs[-1].data_received_cb.add_callback(
            self.logging_callback)
        self.scf.cf.log.add_config(self.logging_configs[-1])

    def clear_logging_configs(self):
        self.stop_logging()
        self.logging_configs = []
        self.scf.cf.log.refresh_toc(
            self.done_refreshing_toc, self.scf.cf.log._toc_cache)
        while self.param_callback_count < 1:
            time.sleep(0.01)
        self.param_callback_count = 0

        self.scf.wait_for_params()

    def done_refreshing_toc(self, *args):

        self.param_callback_count = 1

    def start_logging(self):
        for i in range(0, len(self.logging_configs)):
            self.logging_configs[i].start()

    def stop_logging(self):
        for i in range(0, len(self.logging_configs)):
            self.logging_configs[i].stop()

    def connect(self, uri: str):
        cflib.crtp.init_drivers()
        self.scf = SyncCrazyflie(uri, cf=Crazyflie(rw_cache='./cache'))
        self.scf.open_link()
        self.scf.wait_for_params()

    def disconnect(self):
        if self.is_connected:
            self.scf.close_link()

    @staticmethod
    def list_available_crazyflies():
        cflib.crtp.init_drivers()  # run this again just in case you plug the
        # dongle in
        return cflib.crtp.scan_interfaces()
