import numpy as np
import pyqtgraph as pg


class PlotCanvas(pg.PlotWidget):
    def __init__(self, parent=None, num_axes=5, width=5, height=4, dpi=100):
        super().__init__(parent=parent, width=width, height=height, dpi=dpi)
        self.setLabel('bottom', 'Time (s)')
        self.setLabel('left', 'Amplitude')
        self.setLabel('top', 'Sine Wave Plot')
        self._num_axes = num_axes

        self.current_iter = 0

        # Set the background color of the plot widget to white
        self.setBackground('w')

        if num_axes > 0:
            # Set up the sine wave data and plot
            self.plot_data_ys = np.zeros((1000, num_axes))
            self.plot_data_xs = np.zeros((1000, num_axes))

            self.plot_curves = \
                [self.plot(self.plot_data_xs[:, i], self.plot_data_ys[:, i],
                           pen=pg.mkPen(pg.intColor(i), width=2), symbol='o',
                           symbolSize=5) for i in range(num_axes)]

    def update_num_axes(self):
        num_axes = 0
        if num_axes < 0:
            raise ValueError("num_axes must be greater than 0!")

        if num_axes != self._num_axes:
            if num_axes == 0:  # turn it all off
                self.clear()
            else:
                pass

    def update_plot(self, input_data: float, input_timestamp: float,
                    input_axis: int):
        # Shift the data and add a new point
        self.plot_data_ys[:-1, input_axis] = self.plot_data_ys[1:, input_axis]
        self.plot_data_ys[-1, input_axis] = input_data

        self.plot_data_xs[:-1, input_axis] = self.plot_data_xs[1:, input_axis]
        self.plot_data_xs[-1, input_axis] = input_timestamp

        self.plot_curves[input_axis].setData(self.plot_data_xs[:, input_axis],
                                             self.plot_data_ys[:, input_axis])

        if max(self.plot_data_xs[-1]) < 10:
            self.setXRange(0, 10)  # had padding=0 on both these ranges in
            # case that's needed
        else:
            self.setXRange(max(self.plot_data_xs[-1])-10,
                           max(self.plot_data_xs[-1]))
