import sys
from PyQt5.QtWidgets import QApplication, QMainWindow, QTabWidget

import uCartCommander
from CrazyflieProtoConnection import CrazyflieProtoConnection
from ControlWindow import ControlWindow
from SetpointHandler import SetpointHandler
from GamepadWizard import InputConfigDialogue
from cfclient.utils.input import JoystickReader
from ParameterWindow import ParameterWindow
from LoggingConfigWindow import LoggingConfigWindow
import time


class PyCroCart(QMainWindow):
    def __init__(self, cf: CrazyflieProtoConnection,
                 setpoint_handler: SetpointHandler):
        super().__init__()

        self.joystick_reader = JoystickReader()

        self.tabs = QTabWidget()
        self.tab1 = ControlWindow(cf.logging_queue, setpoint_handler,
                                  self.joystick_reader, cf)
        self.tab2 = InputConfigDialogue(self.joystick_reader,
                                        self.tab1.setpoint_menu.enableGamepad)
        self.tab3 = ParameterWindow(cf)
        self.tab4 = LoggingConfigWindow(
            cf, self.tab1.logging_menu.update_available_logging_variables)

        self.setGeometry(100, 200, 600, 600)

        self.tabs.addTab(self.tab1, "Controls Window")
        self.tabs.addTab(self.tab2, "Gamepad Configuration")
        self.tabs.addTab(self.tab3, "Parameter Window")
        self.tabs.addTab(self.tab4, "Logging Window")

        self.setCentralWidget(self.tabs)
        # self.show()


# Start the application
if __name__ == '__main__':

    app = QApplication(sys.argv)

    cf1 = CrazyflieProtoConnection()
    uri = 'radio://0/75/2M/E7E7E7E7E7'
    cf1.connect(uri)
    cf1.scf.cf.commander = uCartCommander.Commander(cf1.scf.cf)

    time.sleep(1)
    setpoint_handler1 = SetpointHandler(cf1.scf.cf.commander, cf1)

    window = PyCroCart(cf1, setpoint_handler1)
    window.show()

    sys.exit(app.exec_())
