from enum import Enum

from PyQt5.QtCore import QTimer

from uCartCommander import Commander
from threading import Semaphore


class FlightMode(Enum):
    TYPE_STOP: int = 0
    TYPE_VELOCITY_WORLD: int = 1
    TYPE_ZDISTANCE: int = 2
    altHoldType: int = 4
    TYPE_HOVER: int = 5
    FULL_STATE_TYPE: int = 6
    TYPE_POSITION: int = 7
    # Custom modes -----------
    ATTITUDE_RATE_TYPE: int = 8
    ATTITUDE_TYPE: int = 9
    MIXED_ATTITUDE_TYPE: int = 10


class Setpoint:

    def __init__(self):
        self.yaw: float = 0
        self.pitch: float = 0
        self.roll: float = 0
        self.thrust: int = 0


class SetpointHandler:

    def __init__(self, commander: Commander, cf):

        self.setpoint = Setpoint()
        self.commander = commander
        self.cf = cf

        self.setpoint_semaphore = Semaphore(1)
        self._flight_mode = FlightMode.TYPE_STOP

        # Set up the timer to update the plot
        self.timer = QTimer()
        self.timer.timeout.connect(self.update)
        self.timer.start(500)

    def update(self):
        if self._flight_mode != FlightMode.TYPE_STOP:
            self.sendSetpoint()

    def getFlightMode(self):
        return self._flight_mode

    def setAttitudeMode(self):
        self.setpoint_semaphore.acquire()
        self._flight_mode = FlightMode.ATTITUDE_TYPE
        self.setpoint_semaphore.release()

    def setRateMode(self):
        self.setpoint_semaphore.acquire()
        self._flight_mode = FlightMode.ATTITUDE_RATE_TYPE
        self.setpoint_semaphore.release()

    def startFlying(self):
        self.commander.start_flying()

    def stopFlying(self):

        self.setpoint_semaphore.acquire()
        self._flight_mode = FlightMode.TYPE_STOP

        self.commander.send_notify_setpoint_stop(0)
        self.setpoint_semaphore.release()
        print("Stop flying")

    def setSetpoint(self, yaw: float, pitch: float, roll: float, thrust: int):

        self.setpoint_semaphore.acquire()

        self.setpoint.yaw = yaw
        self.setpoint.pitch = pitch
        self.setpoint.roll = roll
        self.setpoint.thrust = thrust
        self.sendSetpointUnsafe()

        self.setpoint_semaphore.release()

    def sendSetpoint(self):

        self.setpoint_semaphore.acquire()

        if self._flight_mode == FlightMode.ATTITUDE_TYPE:

            thrust = self.setpoint.thrust * 65000 / 100
            print(f"Set attitude: {self.setpoint.yaw}, {self.setpoint.pitch}, "
                  f"{self.setpoint.roll}, {thrust}")

            self.commander.send_attitude_setpoint(
                self.setpoint.yaw, self.setpoint.pitch, self.setpoint.roll, 
                thrust)

        elif self._flight_mode == FlightMode.ATTITUDE_RATE_TYPE:

            thrust = self.setpoint.thrust * 65000 / 100
            print(f"Set attitude rate: {self.setpoint.yaw},"
                  f" {self.setpoint.pitch}, "
                  f"{self.setpoint.roll}, {thrust}")

            self.commander.send_attitude_rate_setpoint(
                self.setpoint.yaw, self.setpoint.pitch, self.setpoint.roll,
                thrust)

        self.setpoint_semaphore.release()

    def sendSetpointUnsafe(self):
        print("Unsafe mode activate :)")
        if self._flight_mode == FlightMode.ATTITUDE_TYPE:

            thrust = self.setpoint.thrust * 65000 / 100
            print(f"Set attitude: {self.setpoint.yaw}, {self.setpoint.pitch}, "
                  f"{self.setpoint.roll}, {self.setpoint.thrust}")

            self.commander.send_attitude_setpoint(
                self.setpoint.yaw, self.setpoint.pitch, self.setpoint.roll,
                int(thrust))

        elif self._flight_mode == FlightMode.ATTITUDE_RATE_TYPE:

            thrust = self.setpoint.thrust * 65000 / 100
            print(f"Set attitude rate: {self.setpoint.yaw},"
                  f" {self.setpoint.pitch}, "
                  f"{self.setpoint.roll}, {thrust}")

            self.commander.send_attitude_rate_setpoint(
                self.setpoint.yaw, self.setpoint.pitch, self.setpoint.roll,
                thrust)
